@@ -35,13 +35,15 @@ __credits__ = [
     'Nikolaos Koutsopoulos <nhk@mochimedia.com>',
     'Julian Krause <juliank@yelp.com>',
     'Robert Leftwich <rl.0x0@eml.cc>',
+    'Tetsuya Morimoto <tetsuya.morimoto@gmail.com>',
     'Wahbeh Qardaji <wqardaji@yelp.com>',
     'Jimmy Retzlaff <jretz@yelp.com>',
     'Ned Rockson <ned@tellapart.com>',
+    'Paul Scott <paul@duedil.com>',
     'Steve Spencer <steve@bigfrog.net>',
     'Jyry Suvilehto <jyry.suvilehto@iki.fi>',
     'Matthew Tai <mtai@adku.com>',
     'Paul Wais <pwais@yelp.com>',
 ]
 
-__version__ = '0.3.3.1'
+__version__ = '0.3.4.1'
