@@ -30,9 +30,11 @@ from mrjob.conf import combine_local_envs
 from mrjob.parse import find_python_traceback
 from mrjob.parse import parse_mr_job_stderr
 from mrjob.runner import MRJobRunner
+from mrjob.runner import RunnerOptionStore
 from mrjob.util import cmd_line
 from mrjob.util import read_input
 from mrjob.util import unarchive
+from mrjob.util import is_ironpython
 
 
 log = logging.getLogger('mrjob.local')
@@ -41,6 +43,20 @@ DEFAULT_MAP_TASKS = 2
 DEFAULT_REDUCE_TASKS = 2
 
 
+class LocalRunnerOptionStore(RunnerOptionStore):
+
+    COMBINERS = combine_dicts(RunnerOptionStore.COMBINERS, {
+        'cmdenv': combine_local_envs,
+    })
+
+    def default_options(self):
+        super_opts = super(LocalRunnerOptionStore, self).default_options()
+        return combine_dicts(super_opts, {
+            # prefer whatever interpreter we're currently using
+            'python_bin': [sys.executable or 'python'],
+        })
+
+
 class LocalMRJobRunner(MRJobRunner):
     """Runs an :py:class:`~mrjob.job.MRJob` locally, for testing
     purposes.
@@ -77,6 +93,8 @@ class LocalMRJobRunner(MRJobRunner):
 
     alias = 'local'
 
+    OPTION_STORE_CLASS = LocalRunnerOptionStore
+
     def __init__(self, **kwargs):
         """Arguments to this constructor may also appear in :file:`mrjob.conf`
         under ``runners/local``.
@@ -109,21 +127,6 @@ class LocalMRJobRunner(MRJobRunner):
         # running the job)
         self._internal_jobconf = {}
 
-    @classmethod
-    def _default_opts(cls):
-        """A dictionary giving the default value of options."""
-        return combine_dicts(super(LocalMRJobRunner, cls)._default_opts(), {
-            # prefer whatever interpreter we're currently using
-            'python_bin': [sys.executable or 'python'],
-        })
-
-    @classmethod
-    def _opts_combiners(cls):
-        # on windows, PYTHONPATH should use ;, not :
-        return combine_dicts(
-            super(LocalMRJobRunner, cls)._opts_combiners(),
-            {'cmdenv': combine_local_envs})
-
     # options that we ignore because they require real Hadoop
     IGNORED_HADOOP_OPTS = [
         'hadoop_extra_args',
@@ -304,6 +307,12 @@ class LocalMRJobRunner(MRJobRunner):
         file_names = {}
         input_paths_to_split = []
 
+        # Each file is assigned a 'task number' as if coming from some previous
+        # task. The task number is used to choose the split file name, and
+        # sometimes the file name of the sorted split. This is done so that
+        # when the output files are combined after the final step, they are in
+        # sorted order due to already being lexicographically sorted.
+
         for input_path in input_paths:
             for path in self.ls(input_path):
                 if path.endswith('.gz'):
@@ -311,6 +320,7 @@ class LocalMRJobRunner(MRJobRunner):
                     file_names[path] = {
                         'orig_name': path,
                         'start': 0,
+                        'task_num': len(file_names),
                         'length': os.stat(path)[stat.ST_SIZE],
                     }
                     # this counts as "one split"
@@ -340,11 +350,13 @@ class LocalMRJobRunner(MRJobRunner):
         # Helper functions:
         def create_outfile(orig_name='', start=''):
             # create a new output file and initialize its properties dict
+            task_num = len(file_names)
             outfile_name = os.path.join(tmp_directory,
-                                        'input_part-%05d' % len(file_names))
+                                        'input_part-%05d' % task_num)
             new_file = {
                 'orig_name': orig_name,
                 'start': start,
+                'task_num': task_num,
             }
             file_names[outfile_name] = new_file
             return outfile_name
@@ -372,31 +384,35 @@ class LocalMRJobRunner(MRJobRunner):
 
             # initialize file and accumulators
             outfile_name = create_outfile(path, 0)
-            outfile = open(outfile_name, 'w')
             bytes_written = 0
             total_bytes = 0
+            outfile = None
 
-            # write each line to a file as long as we are within the limit
-            # (split_size)
-            for line_group in line_group_generator(path):
-                if bytes_written >= split_size:
-                    # new split file if we exceeded the limit
-                    file_names[outfile_name]['length'] = bytes_written
-                    total_bytes += bytes_written
+            try:
+                outfile = open(outfile_name, 'w')
 
+                # write each line to a file as long as we are within the limit
+                # (split_size)
+                for line_group in line_group_generator(path):
+                    if bytes_written >= split_size:
+                        # new split file if we exceeded the limit
+                        file_names[outfile_name]['length'] = bytes_written
+                        total_bytes += bytes_written
+
+                        outfile_name = create_outfile(path, total_bytes)
+                        outfile.close()
+                        outfile = open(outfile_name, 'w')
+
+                        bytes_written = 0
+
+                    for line in line_group:
+                        outfile.write(line)
+                        bytes_written += len(line)
+
+                file_names[outfile_name]['length'] = bytes_written
+            finally:
+                if not outfile is None:
                     outfile.close()
-                    outfile_name = create_outfile(path, total_bytes)
-                    outfile = open(outfile_name, 'w')
-
-                    bytes_written = 0
-
-                for line in line_group:
-                    outfile.write(line)
-                    bytes_written += len(line)
-
-            file_names[outfile_name]['length'] = bytes_written
-
-            outfile.close()
 
         return file_names
 
@@ -442,7 +458,12 @@ class LocalMRJobRunner(MRJobRunner):
         all_proc_dicts = []
         self._prev_outfiles = []
 
-        for task_num, file_name in enumerate(file_splits):
+        # The correctly-ordered list of task_num, file_name pairs
+        file_tasks = sorted([
+            (t['task_num'], file_name) for file_name, t
+            in file_splits.items()], key=lambda t: t[0])
+
+        for task_num, file_name in file_tasks:
 
             # setup environment variables
             if step_type == 'M':
@@ -498,9 +519,13 @@ class LocalMRJobRunner(MRJobRunner):
             (translate_jobconf(k, version).replace('.', '_'), str(v))
             for (k, v) in internal_jobconf.iteritems())
 
+        ironpython_env = {'IRONPYTHONPATH': os.getcwd()} if is_ironpython \
+                         else {}
+
         # keep the current environment because we need PATH to find binaries
         # and make PYTHONPATH work
         return combine_local_envs({'PYTHONPATH': os.getcwd()},
+                                  ironpython_env,
                                   os.environ,
                                   jobconf_env,
                                   internal_jobconf_env,
@@ -594,7 +619,6 @@ class LocalMRJobRunner(MRJobRunner):
         log.info('writing to %s' % outfile)
 
         self._prev_outfiles.append(outfile)
-        write_to = open(outfile, 'w')
 
         with open(outfile, 'w') as write_to:
             if combiner_args:
